import 'dart:developer';

import 'package:hydrated_bloc/hydrated_bloc.dart';

import '../../loadableState/loading_error.dart';
import '../../repository/repository.dart';
import 'loadable_hydrated_bloc_event.dart';
import '../../loadableState/loadable_state.dart';
import 'loadable_save_context.dart';

abstract class LoadableHydratedBloc<
  TEvent extends LoadableHydratedBlocEvent<TState>,
  TState,
  TRepository extends Repository<TState>
> extends HydratedBloc<
    LoadableHydratedBlocEvent<TState>,
    LoadableState<TState>
> {
  late TRepository _repository;
  LoadableHydratedBloc() : super(const LoadableState(
    error: null,
    data: null,
    isLoading: true,
    lastFetch: null,
    reFetch: null,
  )) {

    on<Emit<TState>>((event, emit) {
      emit(LoadableState(
        isLoading: state.isLoading,
        data: event.state(innerState ?? fromNothing()),
        lastFetch: state.lastFetch,
        reFetch: retry,
        error: state.error,
      ));
    });

    on<DataGathered<TState>>((event, emit) => emit(LoadableState(
      isLoading: false,
      data: event.state(innerState ?? fromNothing()),
      lastFetch: DateTime.now().millisecondsSinceEpoch,
      reFetch: retry,
      error: null,
    )));

    on<RefetchStarted<TState>>((event, emit) => emit(LoadableState(
        isLoading: true,
        data: innerState,
        lastFetch: state.lastFetch,
        reFetch: null,
        error: null,
    )));

    on<Error<TState>>((event, emit) => emit(LoadableState(
        isLoading: false,
        data: innerState,
        lastFetch: state.lastFetch,
        reFetch: retry,
        error: event.error
    )));

    _repository = repository();
    fetch();
  }

  TState? get innerState => state.data;
  TRepository get repo => _repository;

  void retry() {
    log('Fetch retry triggered for ${TState.toString()}');
    add(RefetchStarted<TState>());
    fetch();
  }

  void fetch() {
    log('Fetching data for ${TState.toString()}');
    gatherData().catchError(
      (e) {
        log('Error while fetching ${TState.toString()}: ${e.toString()}');
        add(Error(LoadingError(
          message: e.message ?? e.toString(),
          allowRetry: true,
        )));
      },
    ).then((value) {
      log('Fetch for ${TState.toString()} completed!');
    });
  }

  @override
  fromJson(Map<String, dynamic> json) {
    var rawData = LoadableSaveContext.unwrap(json);
    return LoadableState(
      isLoading: true,
      data: fromStorage(rawData.data),
      lastFetch: rawData.meta.timestamp,
      reFetch: null,
      error: null,
    );
  }

  @override
  Map<String, dynamic>? toJson(LoadableState<TState> state) {
    Map<String, dynamic>? data;
    try {
      data = toStorage(state.data);
    } catch(e) {
      log('Failed to save state ${TState.toString()}: ${e.toString()}');
      data = null;
    }

    return LoadableSaveContext.wrap(
      data,
      state.lastFetch ?? DateTime.now().millisecondsSinceEpoch
    );
  }

  Future<void> gatherData();
  TRepository repository();

  TState fromNothing();
  TState fromStorage(Map<String, dynamic> json);
  Map<String, dynamic>? toStorage(TState state);
}
